﻿using Microsoft.Extensions.Logging;
using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.BusinessLogicsContracts;
using PlumbingRepairContracts.DI;

namespace PlumbingRepair
{
	public partial class FormMain : Form
	{
		private readonly ILogger _logger;
		private readonly IOrderLogic _orderLogic;

		private readonly IReportLogic _reportLogic;
		private readonly IWorkProcess _workProcess;
		private readonly IBackUpLogic _backUpLogic;
		public FormMain(ILogger<FormMain> logger, IBackUpLogic backUpLogic, IWorkProcess workProcess, IOrderLogic orderLogic, IReportLogic reportLogic)
		{
			InitializeComponent();
			_logger = logger;
			_orderLogic = orderLogic;
			_reportLogic = reportLogic;
			_workProcess = workProcess;
			_backUpLogic = backUpLogic;
		}

		private void FormMain_Load(object sender, EventArgs e)
		{
			LoadData();
		}

		private void LoadData()
		{
			_logger.LogInformation("Загрузка заказов");

			try
			{
				DataGridView.FillandConfigGrid(_orderLogic.ReadList(null));
				_logger.LogInformation("Загрузка заказов");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки заказов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void КомпонентыToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var form = DependencyManager.Instance.Resolve<FormComponents>();
			form.ShowDialog();
		}

		private void ИзделияToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var form = DependencyManager.Instance.Resolve<FormWorks>();
			form.ShowDialog();
		}
		private void КлиентыToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var form = DependencyManager.Instance.Resolve<FormClients>();
			form.ShowDialog();
		}

		private void CreateOrderButton_Click(object sender, EventArgs e)
		{
			var form = DependencyManager.Instance.Resolve<FormCreateOrder>();
			form.ShowDialog();
			LoadData();
		}

		private void IssuedOrderButton_Click(object sender, EventArgs e)
		{
			if (DataGridView.SelectedRows.Count == 1)
			{
				int id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'Выдан'", id);

				try
				{
					var operationResult = _orderLogic.DeliveryOrder(new OrderBindingModel
					{
						Id = id
					});

					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}

					_logger.LogInformation("Заказ №{id} выдан", id);
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка отметки о выдачи заказа");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}

		private void UpdateListButton_Click(object sender, EventArgs e)
		{
			LoadData();
		}

		private void WorksToolStripMenuItem_Click(object sender, EventArgs e)
		{
			using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
			if (dialog.ShowDialog() == DialogResult.OK)
			{
				_reportLogic.SaveWorksToWordFile(new ReportBindingModel
				{
					FileName = dialog.FileName
				});
				MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}

		private void WorkComponentToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var form = DependencyManager.Instance.Resolve<FormReportWorkComponents>();
			form.ShowDialog();
		}

		private void OrdersToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var form = DependencyManager.Instance.Resolve<FormReportOrders>();
			form.ShowDialog();
		}

		private void запускРаботToolStripMenuItem_Click(object sender, EventArgs e)
		{
			_workProcess.DoWork(DependencyManager.Instance.Resolve<IImplementerLogic>(), _orderLogic);
			MessageBox.Show("Процесс обработки запущен", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
		}

		private void исполнителиToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var form = DependencyManager.Instance.Resolve<FormImplementers>();
			form.ShowDialog();
		}

		private void письмаToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var form = DependencyManager.Instance.Resolve<FormMails>();
			form.ShowDialog();
		}

		private void создатьБэкапToolStripMenuItem_Click(object sender, EventArgs e)
		{
			try
			{
				if (_backUpLogic != null)
				{
					var fbd = new FolderBrowserDialog();
					if (fbd.ShowDialog() == DialogResult.OK)
					{
						_backUpLogic.CreateBackUp(new BackUpSaveBindingModel
						{
							FolderName = fbd.SelectedPath
						});
						MessageBox.Show("Бекап создан", "Сообщение",
					   MessageBoxButtons.OK, MessageBoxIcon.Information);
					}
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
	}
}
