﻿using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.SearchModels;
using PlumbingRepairContracts.StoragesContracts;
using PlumbingRepairContracts.ViewModels;
using PlumbingRepairDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace PlumbingRepairDatabaseImplement.Implements
{
    public class ClientStorage : IClientStorage
    {
        public List<ClientViewModel> GetFullList()
        {
            using var context = new PlumbingRepairDataBase();
            return context.Clients
                    .Select(x => x.GetViewModel)
                    .ToList();
        }
        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            if (model == null)
            {
                return new();
            }
			if (model.Id.HasValue)
			{
				var res = GetElement(model);
				return res != null ? new() { res } : new();
			}
			if (!string.IsNullOrEmpty(model.Email))
            {
                using var context = new PlumbingRepairDataBase();
                return context.Clients
                    .Where(x => x.Email.Contains(model.Email))
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return new();
        }
        public ClientViewModel? GetElement(ClientSearchModel model)
        {
			if (string.IsNullOrEmpty(model.ClientFIO) &&
				string.IsNullOrEmpty(model.Email) &&
				string.IsNullOrEmpty(model.Password) &&
				!model.Id.HasValue)
			{
				return null;
			}
			using var context = new PlumbingRepairDataBase();
			var temp = context.Clients
			.FirstOrDefault(x => (string.IsNullOrEmpty(model.ClientFIO) || x.ClientFIO == model.ClientFIO) &&
								 (string.IsNullOrEmpty(model.Email) || x.Email == model.Email) &&
								 (string.IsNullOrEmpty(model.Password) || x.Password == model.Password) &&
								 (!model.Id.HasValue || x.Id == model.Id))
			?.GetViewModel;
			return temp;
		}
        public ClientViewModel? Insert(ClientBindingModel model)
        {
            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }
            using var context = new PlumbingRepairDataBase();
            context.Clients.Add(newClient);
            context.SaveChanges();
            return newClient.GetViewModel;
        }
        public ClientViewModel? Update(ClientBindingModel model)
        {
            using var context = new PlumbingRepairDataBase();
            var client = context.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (client == null)
            {
                return null;
            }
            client.Update(model);
            context.SaveChanges();
            return client.GetViewModel;
        }
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            using var context = new PlumbingRepairDataBase();
            var element = context.Clients
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Clients.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
