﻿using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.SearchModels;
using PlumbingRepairContracts.StoragesContracts;
using PlumbingRepairContracts.ViewModels;
using PlumbingRepairDatabaseImplement.Models;

namespace PlumbingRepairDatabaseImplement.Implements
{
	public class MessageInfoStorage : IMessageInfoStorage
	{
		public MessageInfoViewModel? GetElement(MessageInfoSearchModel model)
		{
			if (string.IsNullOrEmpty(model.MessageId))
			{
				return null;
			}
			using var context = new PlumbingRepairDataBase();
			return context.Messages.FirstOrDefault(x => x.MessageId == model.MessageId)?.GetViewModel;
		}

		public List<MessageInfoViewModel> GetFilteredList(MessageInfoSearchModel model)
		{
			if (!model.ClientId.HasValue)
				return new();
			using var context = new PlumbingRepairDataBase();
			return context.Messages
				.Where(x => x.ClientId == model.ClientId)
				.Select(x => x.GetViewModel)
				.ToList();
		}

		public List<MessageInfoViewModel> GetFullList()
		{
			using var context = new PlumbingRepairDataBase();
			return context.Messages
				.Select(x => x.GetViewModel)
				.ToList();
		}

		public MessageInfoViewModel? Insert(MessageInfoBindingModel model)
		{
			var newMessage = Message.Create(model);
			if (newMessage == null)
			{
				return null;
			}
			using var context = new PlumbingRepairDataBase();
			context.Messages.Add(newMessage);
			context.SaveChanges();
			return newMessage.GetViewModel;
		}
	}
}
