﻿using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.ViewModels;
using PlumbingRepairDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.Runtime.Serialization;

namespace PlumbingRepairDatabaseImplement.Models
{
	[DataContract]
	public class Message : IMessageInfoModel
	{
		[Key]
		[DataMember]
		public string MessageId { get; private set; } = string.Empty;
		[DataMember]
		public int? ClientId { get; private set; }
		[DataMember]
		public string SenderName { get; private set; } = string.Empty;
		[DataMember]
		public DateTime DateDelivery { get; private set; } = DateTime.SpecifyKind(DateTime.Now, DateTimeKind.Utc);
		[DataMember]
		public string Subject { get; private set; } = string.Empty;
		[DataMember]
		public string Body { get; private set; } = string.Empty;

		public virtual Client? Client { get; set; }

		public static Message? Create(MessageInfoBindingModel model)
		{
			if (model == null)
			{
				return null;
			}
			return new()
			{
				Body = model.Body,
				Subject = model.Subject,
				ClientId = model.ClientId,
				MessageId = model.MessageId,
				SenderName = model.SenderName,
				DateDelivery = DateTime.SpecifyKind(model.DateDelivery, DateTimeKind.Utc)
			};
		}

		public MessageInfoViewModel GetViewModel => new()
		{
			Body = Body,
			Subject = Subject,
			ClientId = ClientId,
			MessageId = MessageId,
			SenderName = SenderName,
			DateDelivery = DateDelivery
		};

		public int Id => throw new NotImplementedException();
	}
}
