﻿using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.ViewModels;
using PlumbingRepairDataModels.Enums;
using PlumbingRepairDataModels.Models;
using System.Runtime.Serialization;
using System.Xml.Linq;

namespace PlumbingRepairFileImplement.Models
{
	[DataContract]
	public class Order : IOrderModel
    {
		[DataMember]
		public int Id { get; private set; }
		[DataMember]
		public int WorkId { get; private set; }
		[DataMember]
		public int ClientId { get; private set; }
		[DataMember]
		public int? ImplementerId { get; private set; }
		[DataMember]
		public string WorkName { get; private set; } = string.Empty;
		[DataMember]
		public int Count { get; private set; }
		[DataMember]
		public double Sum { get; private set; }
		[DataMember]
		public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
		[DataMember]
		public DateTime DateCreate { get; private set; } = DateTime.Now;
		[DataMember]
		public DateTime? DateImplement { get; private set; }

        public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                WorkId = model.WorkId,
                WorkName = model.WorkName,
                ClientId = model.ClientId,
				ImplementerId = model.ImplementerId,
				Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }

			DateTime.TryParse(element.Element("DateImplement")!.Value, out DateTime dateImpl);

			var order = new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                WorkId = Convert.ToInt32(element.Element("WorkId")!.Value),
                WorkName = element.Element("WorkName")!.Value,
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
				ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
				Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
				Status = (OrderStatus)Convert.ToInt32(element.Element("Status")!.Value),
				DateCreate = DateTime.ParseExact(element.Element("DateCreate")!.Value, "G", null),
				DateImplement = dateImpl
			};

            return order;
        }

        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
			ImplementerId = model.ImplementerId;
		}

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            WorkId = WorkId,
            WorkName = WorkName,
            ClientId = ClientId,
			ImplementerId = ImplementerId,
			Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };

        public XElement GetXElement => new("Order",
            new XAttribute("Id", Id),
            new XElement("WorkName", WorkName),
            new XElement("WorkId", WorkId.ToString()),
            new XElement("ClientId", ClientId.ToString()),
			new XElement("ImplementerId", ImplementerId),
			new XElement("Count", Count.ToString()),
            new XElement("Sum", Sum.ToString()),
            new XElement("Status", Status.ToString()),
            new XElement("DateCreate", DateCreate.ToString()),
            new XElement("DateImplement", DateImplement.ToString()));
    }
}
